@@ -6,50 +6,35 @@ from xml.dom import minidom, Node
 class cpuconfig:
 	DEF_PAD = 5
 
-	f = open("/proc/deviceinfo/product","r")
+	f = open("/proc/cpuinfo","r")
 	MODEL = f.read()
 	f.close()
 
 	MODEL = MODEL.split();
 
-	MODEL = MODEL[0]
+	MODEL = MODEL[2]
+
+	CPU = MODEL.split("-");
+
+	CPU = CPU[1]
 
 	print MODEL
 	
-	CPU = "Unknown"
+	if (CPU == "PXA250") |(CPU == "PXA255"):
+	   print "Not implemented for " +CPU+ " need testers before!"
+	elif CPU == "PXA270":
+	   KERNEL_POINT = "/sys/devices/system/cpu/cpu0/cpufreq/"
+	   KRN_AVAL_FREQS = KERNEL_POINT + "scaling_available_frequencies"
+	   KRN_FREQ = KERNEL_POINT + "scaling_cur_freq"
+	   KRN_VOLT = KERNEL_POINT + "cpu-voltage"
+	   FREQ_TAB    = ["104", "208", "312", "416", "520", "624"]
+	   CCCR_L_TAB  = [  "8",  "16",  "16",  "16",  "16",  "16"]
+	   CCCR_2N_TAB = [  "2",   "2",   "3",   "4",   "5",   "6"]
 	
-	if MODEL == "SL-C700":
-	    CPU = "PXA250"
-	    CCCR_FILE  = "/proc/cccr_change"
-	elif (MODEL == "SL-C750") | (MODEL == "SL-C7500"):
-	    CPU = "PXA255"
-	    CCCR_FILE  = "/proc/cccr_change"
-	elif (MODEL == "SL-C760") | (MODEL == "SL-C860"):
-	    CPU = "PXA255"
-	    CCCR_FILE  = "/proc/cccr_change"
-	elif (MODEL == "SL-6000"):
-	    CPU = "PXA255"
-	    CCCR_FILE  = "/proc/cccr_change"
-	elif (MODEL == "SL-C1000") | (MODEL == "SL-C3100") | (MODEL == "SL-C3000"):
-	    CPU = "PXA270"
-	    CCCR_FILE  = "/proc/zaurus/CCCR"
-	    VCORE_FILE = "/proc/zaurus/VCORE"
-
-	if CPU == "PXA270":
-	    FREQ_TAB  = ["104", "208", "312", "416", "520", "624"]
-	    CCCR_TAB  = ["2000204", "2000206", "2000208", "2000210", "2000214", "2000218"]
-	    VCORE_TAB = ["04", "06", "08", "0a", "0c", "0e"]
-	elif CPU == "PXA250":
-	    FREQ_TAB = ["199", "398", "472"]
-	    CCCR_TAB = ["141", "241", "242"]
-	elif CPU == "PXA255":
-	    FREQ_TAB = ["199", "398", "472"]
-	    CCCR_TAB = ["141", "161", "162"]
-	    #FREQ_TAB = ["199", "398", "472", "531"]
-	    #CCCR_TAB = ["141", "161", "162", "163"]
-
-	CPU_CCCR  = 0
-	CPU_VCORE = 0
+	CPU_FREQ    = 0
+        CPU_CCCR_L  = 0
+	CPU_CCCR_2N = 0
+	CPU_VCORE   = 0
 
 	if os.name == "nt":
 		CFG_FILENAME = ".\\cpuconfig.cfg"
@@ -57,6 +42,12 @@ class cpuconfig:
 		CFG_FILENAME = os.environ['HOME'] + "/Choices/cpuconfig.cfg"
 
 	def __init__(self):
+		if (self.CPU == "PXA250") | (self.CPU == "PXA255"):
+        	 	Warningdialog = gtk.MessageDialog(None,gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_WARNING,gtk.BUTTONS_OK,"Not implemented for " +self.CPU+ " need testers before!")
+                 	Warningdialog.run()
+                 	Warningdialog.destroy()
+                 	sys.exit()
+
 		if (getpass.getuser().lower() != "root"):
 			Warningdialog = gtk.MessageDialog(None,gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_WARNING,gtk.BUTTONS_OK,"You need to be root in order to run this program.")
 			Warningdialog.run()
@@ -110,34 +101,30 @@ class cpuconfig:
 
 		self.load_prefs();
 
-		self.get_sys_freq();
+		self.get_sys_freq();		
 
 		self.find_freq(self.CPU_FREQ)
 		
 		self.window.show_all()
 
 	def get_sys_freq(self):
-	        f = open(self.CCCR_FILE,"r")
+	        f = open(self.KRN_FREQ,"r")
 		val = f.read()
 		f.close()
-		val = val.split();
-		val = val[0]
-		n = 0
-		for cccr in self.CCCR_TAB:
-		    if val.find(cccr) != -1:
-			self.CPU_FREQ = self.FREQ_TAB[n]
-			self.CPU_CCCR = self.CCCR_TAB[n]
-			if self.CPU == "PXA270":
-			    self.CPU_VCORE = self.VCORE_TAB[n]
-			return True
-		    else:
-			n = n + 1
-		return False
+		
+		self.CPU_FREQ = eval(val)/1000;
+		
+		f = open(self.KRN_VOLT,"r")
+		val = f.read()
+		f.close()
+		self.CPU_VCORE = val;		
+		
+		return True
 
 	def find_freq(self, data=None):
 		n = 0
 		for freq in self.FREQ_TAB:
-		    if (freq == data):
+		    if (eval(freq) == data):
 			self.freq_menu.set_active(n)
 			return True
 		    else:
@@ -147,23 +134,21 @@ class cpuconfig:
 
 	def change_freq(self, combobox):
 		n = combobox.get_active()
-		self.CPU_FREQ = self.FREQ_TAB[n]
-		self.CPU_CCCR = self.CCCR_TAB[n]
-		if self.CPU == "PXA270":
-		    self.CPU_VCORE = self.VCORE_TAB[n]
+		self.CPU_FREQ    = self.FREQ_TAB[n]
+		self.CPU_CCCR_L  = self.CCCR_L_TAB[n]
+		self.CPU_CCCR_2N = self.CCCR_2N_TAB[n]
 		return True
 
 	def close_and_update(self,widget):
 		self.save_prefs()
-		#print self.CPU
-		#print self.CPU_FREQ
-		#print self.CPU_CCCR
-		#if self.CPU == "PXA270":
-		#    print self.CPU_VCORE
-		    
-		if self.CPU == "PXA270":
-		    self.write_val(self.VCORE_FILE, self.CPU_VCORE);
-		self.write_val(self.CCCR_FILE, self.CPU_CCCR);
+		
+		CMD = "dvfm " + self.CPU_CCCR_L + " " + self.CPU_CCCR_2N + " 1 2 1"
+		
+		try:
+			retcode = os.system(CMD)
+		except OSError, e:
+			print >>sys.stderr, "Execution failed:", e
+		
 		gtk.main_quit()
 
 	def write_val(self, file, val):
